#include "component_heart.h"

static void _state_set(ComponentHeart* self, ECS* ecs);
static void _multiplier_update(ComponentHeart* self, ECS* ecs);

static void
_multiplier_update(ComponentHeart* self, ECS* ecs)
{
	ComponentGoodies* goodies;

	goodies = ecs_component_get(ecs, ECS_COMPONENT_GOODIES, self->component.id);

	if (self->concurrentUses > 1)
	{
		self->multiplier = self->uses + 1;
		self->multiplier += pow(self->concurrentUses, 2); 
		self->multiplier *= self->multiplierBonus + 1;
	}
	else
	{
		self->multiplier = self->uses + 1;
		self->multiplier *= self->multiplierBonus + 1;
	}

	if (goodies)
		goodies->multiplier = self->multiplier;
}

/* Determines current heart state. */
static void
_state_set(ComponentHeart* self, ECS* ecs)
{
	HeartState previousState;

	previousState = self->state;

	if (self->concurrentUses >= COMPONENT_HEART_CONCURRENT_USES_HEAVY)
		self->state = HEART_STATE_HEAVY;
	else if (self->concurrentUses >= COMPONENT_HEART_CONCURRENT_USES_MEDIUM)
		self->state = HEART_STATE_MEDIUM;
	else
		self->state = HEART_STATE_LIGHT;

	if (self->state != previousState)
	{
		if (self->state == HEART_STATE_HEAVY)
			music_play(&ecs->game->resources.music[MUSIC_COMBAT_THREE], true);
		else if (self->state == HEART_STATE_MEDIUM)
			music_play(&ecs->game->resources.music[MUSIC_COMBAT_TWO], true);
	}
}

/* Ticks heart component. */
void
component_heart_tick(ComponentHeart* self, ECS* ecs)
{
	_multiplier_update(self, ecs);
	_state_set(self, ecs);
}
