#include "component_shop_item.h"

static void _shop_item_bought(ComponentShopItem* self, ECS* ecs);

/* Increases values associated with buying a shop item. */
static void
_shop_item_bought(ComponentShopItem* self, ECS* ecs)
{
	ComponentGoodies* goodies;
	ComponentGoodies* affectedGoodies;
	ComponentUpgrades* upgrades;
	ComponentHealth* health;
	ComponentSprite* sprite;
	vec3 position;

	self->isBought = false;

	goodies = ecs_component_get(ecs, ECS_COMPONENT_GOODIES, self->affectedID);

	if (self->type == SHOP_ITEM_FEED)
		sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, self->affectedID2);
	else
		sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, self->affectedID);

	glm_vec3_copy(sprite->position, position);

	position[1] -= sprite->size[1];
	position[2] += COMPONENT_SHOP_ITEM_ICON_NOTICE_Z_OFFSET;

	if 
	(
	 	(goodies->value < self->price && self->type != SHOP_ITEM_FEED) ||
		(goodies->value == 0 && self->type == SHOP_ITEM_FEED)
	)	
	{
		sound_play(&ecs->game->resources.sounds[SOUND_ERROR], SOUND_NO_PRIORITY);
		return;
	}

	switch (self->type)
	{
		case SHOP_ITEM_UPGRADE_LIFETIME:
			upgrades = ecs_component_get(ecs, ECS_COMPONENT_UPGRADES, self->affectedID);
			upgrades->currentLevels[UPGRADE_LIFETIME]++;
			entity_icon_notice_init(ecs, ecs_entity_add(ecs), position, ICON_COMBAT_TIMER);
			break;
		case SHOP_ITEM_UPGRADE_AMMO:
			upgrades = ecs_component_get(ecs, ECS_COMPONENT_UPGRADES, self->affectedID);
			upgrades->currentLevels[UPGRADE_AMMO]++;
			entity_icon_notice_init(ecs, ecs_entity_add(ecs), position, ICON_AMMO);
			break;
		case SHOP_ITEM_UPGRADE_REDIRECTS:
			upgrades = ecs_component_get(ecs, ECS_COMPONENT_UPGRADES, self->affectedID);
			upgrades->currentLevels[UPGRADE_REDIRECTS]++;
			entity_icon_notice_init(ecs, ecs_entity_add(ecs), position, ICON_REDIRECT);
			break;
		case SHOP_ITEM_HEALTH:
			health = ecs_component_get(ecs, ECS_COMPONENT_HEALTH, self->affectedID);
			if (health->value >= health->max)
			{
				sound_play(&ecs->game->resources.sounds[SOUND_ERROR], SOUND_NO_PRIORITY);
				return;
			}
			health->value++;
			entity_icon_notice_init(ecs, ecs_entity_add(ecs), position, ICON_HEALTH);
			break;
		case SHOP_ITEM_FEED:
			affectedGoodies = ecs_component_get(ecs, ECS_COMPONENT_GOODIES, self->affectedID2);
			affectedGoodies->value += goodies->value;
			entity_icon_notice_init(ecs, ecs_entity_add(ecs), position, ICON_GOODIES);
			break;
		default:
			break;
	}


	if (self->type == SHOP_ITEM_FEED)
	{
		if (goodies->value > 0)
			sound_play(&ecs->game->resources.sounds[SOUND_GULP], SOUND_PRIORITY);
	
		goodies->value = 0;
		return;
	}
	else
		goodies->value -= self->price;

	self->purchaseCount++;

	self->price = self->basePrice * pow((self->purchaseCount + 1), self->exponent);

	sound_play(&ecs->game->resources.sounds[SOUND_UPGRADE], SOUND_SPECIAL);
}

/* Sets shop_item component info. */
void
component_shop_item_init(ComponentShopItem* self, ShopItemType type, u32 price, f32 exponent, u32 affectedID, u32 affectedID2)
{
	self->basePrice = price;
	self->price = self->basePrice;
	self->exponent = exponent;
	self->type = type;
	self->affectedID = affectedID;
	self->affectedID2 = affectedID2;
}

/* Ticks shop item component. */
void
component_shop_item_tick(ComponentShopItem* self, ECS* ecs)
{
	if (self->isBought)
		_shop_item_bought(self, ecs);
}
