#include "entity_beanie.h"

/* Initializes a beanie entity. */
void
entity_beanie_init(ECS* ecs, u32 id, vec3 position, u32 targetID, u32 level)
{
	ComponentEnemy* enemy;
	ComponentActionDeleteOnTouchGameEntityType* actionDeleteOnTouchGameEntityType;
	ComponentAnimationRotate* animationRotate;
	ComponentSprite* sprite;
	ComponentGameEntityType* gameEntityType;
	ComponentPhysics* physics;
	ComponentPhysics* targetPhysics;
	ComponentGameObject* gameObject;
	f32 speed;
	f32 angle;

	enemy = ecs_component_add(ecs, ECS_COMPONENT_ENEMY, id);
	actionDeleteOnTouchGameEntityType = ecs_component_add(ecs, ECS_COMPONENT_ACTION_DELETE_ON_TOUCH_GAME_ENTITY_TYPE, id);
	animationRotate = ecs_component_add(ecs, ECS_COMPONENT_ANIMATION_ROTATE, id);
	gameEntityType = ecs_component_add(ecs, ECS_COMPONENT_GAME_ENTITY_TYPE, id);

	component_enemy_init
	(
		enemy,
		ecs,
		ecs->game->resources.textures[TEXTURE_BEANIE],
		(s32*)BEANIE_FRAME_SIZE,
		(s32*)BEANIE_ATLAS_SIZE,
		(f32*)BEANIE_SIZE,
		(f32*)BEANIE_PHYSICS_SIZE,
		position,
		BEANIE_FRICTION,
		BEANIE_VELOCITY_MAX,
		BEANIE_HEALTH,
		BEANIE_IS_BOUNCE,
		BEANIE_IS_AFFECTED_BY_GRAVITY,
		BEANIE_IS_SOLID,
		BEANIE_GOODIE_COUNT,
		1
	);

	sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, id);

	component_action_delete_on_touch_game_entity_type_init
	(
		actionDeleteOnTouchGameEntityType, 
		BEANIE_ACTION_DELETE_ON_TOUCH_GAME_ENTITY_TYPE
	);

	component_game_entity_type_init
	(
		gameEntityType, 
		GAME_ENTITY_BEANIE
	);

	component_animation_rotate_init(animationRotate, BEANIE_ROTATION_SPEED);

	sprite->rotation = RANDOM_F32(0, TAU);

	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, id);
	targetPhysics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, targetID);
	gameObject = ecs_component_get(ecs, ECS_COMPONENT_GAME_OBJECT, id);

	angle = ATAN
	(
		physics->position[0],
		targetPhysics->position[1],
		physics->position[1],
		targetPhysics->position[1]
	);

	speed = BEANIE_SPEED + ((level - 1) * BEANIE_LEVEL_VELOCITY_MULTIPLIER);
	 
	speed = MAX(speed, BEANIE_VELOCITY_MAX);

	physics->velocity[0] = cos(angle) * speed;
	physics->velocity[1] = sin(angle) * speed;

	gameObject->height = BEANIE_HEIGHT;
}
