#include "results.h"

static void _results_entity_init(Results* self);

/* Initializes entities for the results state. */
static void
_results_entity_init(Results* self)
{
	self->menuButtonID = ecs_entity_add(&self->state->game->ecs);
	self->backgroundID = ecs_entity_add(&self->state->game->ecs);
	self->textID = ecs_entity_add(&self->state->game->ecs);

	entity_sprite_init
	(
		&self->state->game->ecs,
		self->backgroundID,
		self->state->game->resources.textures[TEXTURE_STATE_FADE],
		(f32*)RESULTS_BACKGROUND_SIZE,
		(f32*)RESULTS_BACKGROUND_POSITION
	);

	entity_text_init
	(
		&self->state->game->ecs,
		self->textID,
		&self->state->game->resources.fonts[FONT_SMALL],
		STRING_RESULTS_TEXT,
		strlen(STRING_RESULTS_TEXT),
		-1,
		(f32*)RESULTS_TEXT_POSITION,
		(f32*)COLOR_OPAQUE
	);

	entity_menu_button_init
	(
		&self->state->game->ecs,
		self->menuButtonID,
		(f32*)RESULTS_MENU_BUTTON_POSITION,
		MENU_BUTTON_OK
	);
}

/* Initializes results state. */
void
results_init(Results* self, State* state)
{
	memset(self, '\0', sizeof(Results));

	self->state = state;
	
	_results_entity_init(self);

	music_play(&self->state->game->resources.music[MUSIC_RESULTS], true);
}

/* Ticks results state. */
void
results_tick(Results* self)
{
	ComponentButton* button;

	button = ecs_component_get(&self->state->game->ecs, ECS_COMPONENT_BUTTON, self->menuButtonID);

	if (button->state == BUTTON_PRESSED)
	{
		sound_play(&self->state->game->resources.sounds[SOUND_SELECT], SOUND_PRIORITY);
		self->isChange = true;
	}
}
