#pragma once

#include "play/play.h"
#include "title/title.h"
#include "game_over/game_over.h"
#include "results/results.h"

#include "../resource/resource_shader.h"
#include "../resource/resource_texture.h"
#include "../resource/resource_font.h"
#include "../resource/resource_music.h"
#include "../resource/resource_sound.h"

#include "../ecs/entity/entity_state_fade.h"

static const ShaderType* STATE_SHADERS[STATE_COUNT] =
{
	PLAY_SHADERS,
	TITLE_SHADERS,
	GAME_OVER_SHADERS,
	RESULTS_SHADERS
};

static const u32 STATE_SHADER_COUNT[STATE_COUNT] = 
{
	PLAY_SHADER_COUNT,
	TITLE_SHADER_COUNT,
	GAME_OVER_SHADER_COUNT,
	RESULTS_SHADER_COUNT
};

static const TextureType* STATE_TEXTURES[STATE_COUNT] =
{
	PLAY_TEXTURES,
	TITLE_TEXTURES,
	GAME_OVER_TEXTURES,
	RESULTS_TEXTURES,
};

static const u32 STATE_TEXTURE_COUNT[STATE_COUNT] =
{
	PLAY_TEXTURE_COUNT,
	TITLE_TEXTURE_COUNT,
	GAME_OVER_TEXTURE_COUNT,
	RESULTS_TEXTURE_COUNT,
};

static const FontType* STATE_FONTS[STATE_COUNT] =
{
	PLAY_FONTS,
	TITLE_FONTS,
	GAME_OVER_FONTS,
	RESULTS_FONTS
};

static const u32 STATE_FONT_COUNT[STATE_COUNT] =
{
	PLAY_FONT_COUNT,
	TITLE_FONT_COUNT,
	GAME_OVER_FONT_COUNT,
	RESULTS_FONT_COUNT
};

static const SoundType* STATE_SOUNDS[STATE_COUNT] =
{
	PLAY_SOUNDS,
	TITLE_SOUNDS,
	GAME_OVER_SOUNDS,
	RESULTS_SOUNDS
};

static const u32 STATE_SOUND_COUNT[STATE_COUNT] =
{
	PLAY_SOUND_COUNT,
	TITLE_SOUND_COUNT,
	GAME_OVER_SOUND_COUNT,
	RESULTS_SOUND_COUNT
};

static const MusicType* STATE_MUSIC[STATE_COUNT] =
{
	PLAY_MUSIC,
	TITLE_MUSIC,
	GAME_OVER_MUSIC,
	RESULTS_MUSIC
};

static const u32 STATE_MUSIC_COUNT[STATE_COUNT] =
{
	PLAY_MUSIC_COUNT,
	TITLE_MUSIC_COUNT,
	GAME_OVER_MUSIC_COUNT,
	RESULTS_MUSIC_COUNT
};

void state_change(State* self, StateType type);
void state_init(State* self, Game* game, StateType type);
void state_free(State* self);
void state_tick(State* self);
